﻿#NoEnv  ; Recommended for performance and compatibility with future AutoHotkey releases.
#Warn ;useunsetglobal,off ; Recommended for catching common errors.
SendMode Input  ; Recommended for new scripts due to its superior speed and reliability.
SetWorkingDir %A_ScriptDir%  ; Ensures a consistent starting directory.
#notrayicon
/*
twg.ahk/exe - Terraria World Generator:
- usage: extract twg.exe/ahk and MoreTerra to your TerrariaServer install
- moreterra world viewer/png mapper is only required for image mapping option
- batch limit set to 100 worlds max, saving to .\twg-bat in your default world folder
- overwrites any of its own existing files so remember to save maps you wish to keep
- target dir and batch limit can be changed via default vars 'target', 'vmax'
- for 100 worlds, small = 1.2GB, medium = 2.5GB, large = 5GB!
*/
;default vars
target = %a_mydocuments%\My Games\Terraria\worlds\twg-bat\
vmax := 100
vlen := strlen(vmax)
nmaps :=
tips := {}
tips["nedit"] := "set a number from 1-" . vmax
tips["browse"] := "browse world files"
;handy for debug
if !a_iscompiled
	showtip("loaded: '" a_scriptname "'",3)
;show/hide tooltips
showtip(x,delay := 2) {
	tooltip % x
	delay := (delay * 1000)
	settimer notip,-%delay%
	return
	
	notip:
	tooltip
	return
}
;track mouse, trigger tips
WM_MOUSEMOVE() {
	listlines off	;wow much spam
	global tips
	static curcontrol :=,prevcontrol :=,_tip :=
	curcontrol := a_guicontrol
	for k,v in tips
		if (curcontrol != prevcontrol && k = curcontrol) {
			showtip(v,5)
			prevcontrol := curcontrol
		}
}
;status text animation thingy
sb_anim() {
	static dot :=
	dot := (strlen(dot) < 5) ? (dot . ".") : ""
	return dot
}
;build gui
gui +owndialogs
gui add,text,y20 w125 h20 right section,Worlds to Generate:
gui add,edit,ys-3 w65 hp number limit%vlen% vnedit
gui add,updown,range0-%vmax% vnmaps
gui add,button, ys-3 hp vbrowse gbrowse,...
gui add,text,xm w125 h20 right section,World Size:
gui add,dropdownlist,ys-3 w65 hp center r3 vwsize,small||medium|large
gui add,text,xm h20 w125 h20 right section,Save Image Maps:
gui add,checkbox,ys-3 hp vpng checked
gui add,progress,xm w260 -smooth vpbar
gui add,button,w125 h25 section default vbatch gbatch,Start
gui add,button,ys w125 hp vquit gquit,Exit
gui add,statusbar,,%a_space%Set a number of worlds to build.
gui show,,Terraria World Generator
onmessage(0x200, "WM_MOUSEMOVE")
return
;world gen batch
;assign inputs, check environment
batch:
gui submit,nohide
if !nmaps
	return
;find terrariaserver
ifnotexist terrariaserver.exe 
{
	msgbox TerrariaServer is required to generate worlds.
	nmaps :=
	return
}
;find moreterra
if png
	ifnotexist moreterra.exe
	{
		msgbox MoreTerra is required to save image maps.
		nmaps :=
		return
	}
;set world size, check free space, confirm batch
;approx map size small = 12MB, medium = 25MB, large = 50MB
s := ((wsize = "large") ? 3 : (wsize = "medium") ? 2 : 1)
btotal := (s = 3) ? (nmaps * 50) : (s = 2) ? (nmaps * 25) : (nmaps * 12)
drivespacefree bfree,% substr(target,1,2)
msgbox 1,Continue?,% "This batch will need ~" . btotal . " MB of free space on drive " . substr(target,1,2)
	. "`nFiles will be saved to:`n" . target
	. "`n`nSpace free: " . bfree . " MB"
ifmsgbox cancel
{
	nmaps :=
	return
}
if (bfree <= btotal) {
	sb_settext(" Not enough free drive space.")
	nmaps :=
	return
}
;disable ui controls, start batch
guicontrol,,pbar,0
guicontrol text,nedit,%nmaps%
guicontrol +disabled,nedit
guicontrol +disabled,nmaps
guicontrol +disabled,wsize
guicontrol +disabled,png
guicontrol +disabled,batch
guicontrol text,quit,Abort
guicontrol +default,quit
;set progress increments, pad filenames, loop world gen
perindex := (100 // nmaps)
t := strlen(nmaps), o := (1 - t)
while (nmaps > 0) {
	;set current iteration filename
	n := substr("000" . a_index,o)
	w = %target%%n%.wld
	; >terrariaserver.exe -world worldfile -autocreate size -worldname name
	run terrariaserver.exe -world "%w%" -autocreate %s% -worldname world-%n%,,min useerrorlevel,cpid
	if errorlevel {
		sb_settext(" TerrariaServer could not be run.")
		nmaps :=
		break
	}
	;wait for file write or abort
	fo :=
	listlines off
	while !fo {
		if nmaps {
			sb_settext(" Generating world " . n . sb_anim())
			sleep 250
		} else {
			sb_settext(" Job cancelled.")
			break
		}
		fo := fileopen(w,"r")
	}
	fo.close()
	listlines on
	;break if abort
	if !nmaps
		break
	;quit server and save png
	controlsend,,exit-nosave{enter},ahk_pid %cpid%
	process waitclose,%cpid%
	if png {
		p = %target%%n%.png
		sb_settext(" Saving PNG map...")
		; >moreterra.exe -w source -o output
		runwait moreterra.exe -w "%w%" -o "%p%",,min useerrorlevel
		if errorlevel {
			sb_settext(" MoreTerra could not be run.")
			nmaps :=
			return
		}
	}
	;update progress, decrement
	guicontrol,,pbar,+%perindex%
	nmaps ? nmaps--
}
;job complete, update status
if (nmaps = 0) {
	guicontrol,,pbar,100
	sb_settext(" World generation complete.")
}
;re-enable ui controls
guicontrol enable,nedit
guicontrol enable,nmaps
guicontrol enable,wsize
guicontrol enable,png
guicontrol enable,batch
guicontrol +default,batch
guicontrol text,quit,Exit
return
;browse to world files
browse:
run %a_windir%\explorer.exe %a_mydocuments%\my games\terraria\worlds
return
;handle abort/exit attempts
quit:
guiclose:
;quit console task
if nmaps {
	msgbox 1,Abort,Cancel batch?
	ifmsgbox cancel
		return
	else {
		nmaps :=
		controlsend,,^{c},ahk_pid %cpid%
	}
;or close gui
} else {
	if a_iscompiled
		exitapp 
	else
		gui destroy
}
return
